/* SPDX-License-Identifier: ((GPL-2.0 WITH Linux-syscall-note) OR BSD-3-Clause) */
/* Do not edit directly, auto-generated from: */
/*	Documentation/netlink/specs/nfsd.yaml */
/* YNL-GEN uapi header */
/* To regenerate run: tools/net/ynl/ynl-regen.sh */

#ifndef _UAPI_LINUX_NFSD_NETLINK_H
#define _UAPI_LINUX_NFSD_NETLINK_H

#define NFSD_FAMILY_NAME	"nfsd"
#define NFSD_FAMILY_VERSION	1

enum {
	NFSD_A_RPC_STATUS_XID = 1,
	NFSD_A_RPC_STATUS_FLAGS,
	NFSD_A_RPC_STATUS_PROG,
	NFSD_A_RPC_STATUS_VERSION,
	NFSD_A_RPC_STATUS_PROC,
	NFSD_A_RPC_STATUS_SERVICE_TIME,
	NFSD_A_RPC_STATUS_PAD,
	NFSD_A_RPC_STATUS_SADDR4,
	NFSD_A_RPC_STATUS_DADDR4,
	NFSD_A_RPC_STATUS_SADDR6,
	NFSD_A_RPC_STATUS_DADDR6,
	NFSD_A_RPC_STATUS_SPORT,
	NFSD_A_RPC_STATUS_DPORT,
	NFSD_A_RPC_STATUS_COMPOUND_OPS,

	__NFSD_A_RPC_STATUS_MAX,
	NFSD_A_RPC_STATUS_MAX = (__NFSD_A_RPC_STATUS_MAX - 1)
};

enum {
	NFSD_A_SERVER_THREADS = 1,
	NFSD_A_SERVER_GRACETIME,
	NFSD_A_SERVER_LEASETIME,
	NFSD_A_SERVER_SCOPE,
	NFSD_A_SERVER_MIN_THREADS,

	__NFSD_A_SERVER_MAX,
	NFSD_A_SERVER_MAX = (__NFSD_A_SERVER_MAX - 1)
};

enum {
	NFSD_A_VERSION_MAJOR = 1,
	NFSD_A_VERSION_MINOR,
	NFSD_A_VERSION_ENABLED,

	__NFSD_A_VERSION_MAX,
	NFSD_A_VERSION_MAX = (__NFSD_A_VERSION_MAX - 1)
};

enum {
	NFSD_A_SERVER_PROTO_VERSION = 1,

	__NFSD_A_SERVER_PROTO_MAX,
	NFSD_A_SERVER_PROTO_MAX = (__NFSD_A_SERVER_PROTO_MAX - 1)
};

enum {
	NFSD_A_SOCK_ADDR = 1,
	NFSD_A_SOCK_TRANSPORT_NAME,

	__NFSD_A_SOCK_MAX,
	NFSD_A_SOCK_MAX = (__NFSD_A_SOCK_MAX - 1)
};

enum {
	NFSD_A_SERVER_SOCK_ADDR = 1,

	__NFSD_A_SERVER_SOCK_MAX,
	NFSD_A_SERVER_SOCK_MAX = (__NFSD_A_SERVER_SOCK_MAX - 1)
};

enum {
	NFSD_A_POOL_MODE_MODE = 1,
	NFSD_A_POOL_MODE_NPOOLS,

	__NFSD_A_POOL_MODE_MAX,
	NFSD_A_POOL_MODE_MAX = (__NFSD_A_POOL_MODE_MAX - 1)
};

enum {
	NFSD_CMD_RPC_STATUS_GET = 1,
	NFSD_CMD_THREADS_SET,
	NFSD_CMD_THREADS_GET,
	NFSD_CMD_VERSION_SET,
	NFSD_CMD_VERSION_GET,
	NFSD_CMD_LISTENER_SET,
	NFSD_CMD_LISTENER_GET,
	NFSD_CMD_POOL_MODE_SET,
	NFSD_CMD_POOL_MODE_GET,

	__NFSD_CMD_MAX,
	NFSD_CMD_MAX = (__NFSD_CMD_MAX - 1)
};

#endif /* _UAPI_LINUX_NFSD_NETLINK_H */
