/* SPDX-License-Identifier: (GPL-2.0-only OR MIT) */
/*
 * Copyright (C) 2022 Laurent Pinchart <laurent.pinchart@ideasonboard.com>
 */

#ifndef __DT_BINDINGS_MEDIA_VIDEO_INTERFACES_H__
#define __DT_BINDINGS_MEDIA_VIDEO_INTERFACES_H__

#define MEDIA_BUS_TYPE_CSI2_CPHY		1
#define MEDIA_BUS_TYPE_CSI1			2
#define MEDIA_BUS_TYPE_CCP2			3
#define MEDIA_BUS_TYPE_CSI2_DPHY		4
#define MEDIA_BUS_TYPE_PARALLEL			5
#define MEDIA_BUS_TYPE_BT656			6

#define MEDIA_BUS_CSI2_CPHY_LINE_ORDER_ABC	0
#define MEDIA_BUS_CSI2_CPHY_LINE_ORDER_ACB	1
#define MEDIA_BUS_CSI2_CPHY_LINE_ORDER_BAC	2
#define MEDIA_BUS_CSI2_CPHY_LINE_ORDER_BCA	3
#define MEDIA_BUS_CSI2_CPHY_LINE_ORDER_CAB	4
#define MEDIA_BUS_CSI2_CPHY_LINE_ORDER_CBA	5

#define MEDIA_PCLK_SAMPLE_FALLING_EDGE		0
#define MEDIA_PCLK_SAMPLE_RISING_EDGE		1
#define MEDIA_PCLK_SAMPLE_DUAL_EDGE		2

#endif /* __DT_BINDINGS_MEDIA_VIDEO_INTERFACES_H__ */
