/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_POWER_SUN55I_A523_PCK600_H_
#define _DT_BINDINGS_POWER_SUN55I_A523_PCK600_H_

#define PD_VE			0
#define PD_GPU			1
#define PD_VI			2
#define PD_VO0			3
#define PD_VO1			4
#define PD_DE			5
#define PD_NAND			6
#define PD_PCIE			7

#endif /* _DT_BINDINGS_POWER_SUN55I_A523_PCK600_H_ */
