/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (C) 2024 Inochi Amaoto <inochiama@gmail.com>
 */

#ifndef __DT_BINDINGS_SOPHGO_SG2044_PLL_H__
#define __DT_BINDINGS_SOPHGO_SG2044_PLL_H__

#define CLK_FPLL0			0
#define CLK_FPLL1			1
#define CLK_FPLL2			2
#define CLK_DPLL0			3
#define CLK_DPLL1			4
#define CLK_DPLL2			5
#define CLK_DPLL3			6
#define CLK_DPLL4			7
#define CLK_DPLL5			8
#define CLK_DPLL6			9
#define CLK_DPLL7			10
#define CLK_MPLL0			11
#define CLK_MPLL1			12
#define CLK_MPLL2			13
#define CLK_MPLL3			14
#define CLK_MPLL4			15
#define CLK_MPLL5			16

#endif /* __DT_BINDINGS_SOPHGO_SG2044_PLL_H__ */
